%--------------------------------------------------------------------
% PURPOSE: This is the main file for testing the FRD validity assumptions
% using Arai, Hsu, Kitagawa, Mourifie, and Wan (2020)
% Here we consider the AL1999 data set.
%--------------------------------------------------------------------

diary on
tic
clc
clear


%##########################################################################
rich=15;                    % Parameter Q in the interval class
nbs=1000;                   %BOOTSTRAP REPLICATIONS
vweight=1;                   % vweight=1 is using variance weighting. vweight=0 means no weighting.
smooth=4.5;                       % undersmoothing constan if undersmooth bandwdith are used. 
band={'three','five','AI','IK','CCT'}; % Five bandwidth choices (undersmoothed) reported in the main text
% band={'AI-CER','IK-CER','CCT-CER','AI-RBC','IK-RBC','CCT-RBC'}; 
% other possible choice of bandwidths. XX-CER represents rule of thumb coverage
% rate optimal bandwidth based on XX, and XX-RBC represents MSE-optimal
% bandwidth with robust bias correction based on XX. 
CAT={'g4_math','g4_verb','g5_math','g5_verb'}; % Four subjects: 4_math, g4_verb, g5_math or g5_verb
CUTOFF=[40, 80, 120]; % Three cutoff values
if vweight==0
    XI=1;
else
        XI=[0.00999950,0.03160696,0.17058722,0.5];
%     XI=0.00999950;
end
VW={'no weighting','weigthing'};
nband=length(band);         % # of bandwidths
numcat=4;                   % # of catogories: g.
numcutoff=3;                 % # of cutoff values: 40, 80, 120
numxi=length(XI);             % # of xi parameters to use
pwd;
load('angrist_levy_data.mat')


%#########################################################################
for cat=1:numcat
    file=CAT{cat};
    disp(['subject: ',CAT{cat}])
    Data=eval(file);
    Y=Data(:,1); % outcome variable
    sdy=(Y-mean(Y))./std(Y);
    S=Data(:,3);
    R=Data(:,4); % Running variable
    D=Data(:,2); % Treatment variable
    for cutoff=1:numcutoff
        c=CUTOFF(cutoff)+1;
        I=find(D==cutoff|D==cutoff+1);
        y=sdy(I);
        t=D(I)-cutoff;
        x=R(I);
        s=S(I);
        for tune=1:numxi
            xi=XI(tune);
            pval=nan(1,nband);
            Sn_vec=nan(1,nband);
            CV01=nan(1,nband);
            CV05=nan(1,nband);
            CV10=nan(1,nband);
            n=length(y);
            n_plus=sum((x>=c));
            n_minus=sum((x<c));
            B=zeros(2,4);
            for nb=1:nband
                bandwidth=band(nb);
                if strcmp(bandwidth,'three')==1
                    bw=[3;3];
                elseif strcmp(bandwidth,'five')==1
                    bw=[5;5];
                elseif strcmp(bandwidth,'AI')==1
                    output = rddmmsef(x,y,t,c);
                    bws=[output.h(2),output.h(1)];
                    bw=[min(bws(1)*n_minus^(1/5-1/smooth),15),min(bws(2)*n_plus^(1/5-1/smooth),15)];
                    % Here AI bandwidth is regularized by no larger than 15
                elseif strcmp(bandwidth,'IK')==1
                    bws = rd_optbandwidth_1(y,x,c);
                    bw=[bws*n_minus^(1/5-1/smooth),bws*n_plus^(1/5-1/smooth)];
                elseif strcmp(bandwidth,'CCT')==1
                    output = rddcct(x,y,c);
                    bws=output.h;
                    bw=[bws*n_minus^(1/5-1/smooth),bws*n_plus^(1/5-1/smooth)];
                elseif strcmp(bandwidth,'AI-CER')==1
                    output = rddmmsef(x,y,t,c);
                    bws=[output.h(2),output.h(1)];
                    bw=[bws(1)*n_minus^(-1/20),bws(2)*n_plus^(-1/20)];
                elseif strcmp(bandwidth,'IK-CER')==1
                    bws = rd_optbandwidth_1(y,x,c);
                    bw=[bws*n_minus^(-1/20),bws*n_plus^(-1/20)];
                elseif strcmp(bandwidth,'CCT-CER')==1
                    output = rddcct(x,y,c);
                    bws=output.h;
                    bw=[bws*n_minus^(-1/20),bws*n_plus^(-1/20)];
                elseif strcmp(bandwidth,'AI-RBC')==1
                    output = rddmmsef(x,y,t,c);
                    bws=[output.h(2),output.h(1)];
                    bw=bws;
                elseif strcmp(bandwidth,'IK-RBC')==1
                    bws = rd_optbandwidth_1(y,x,c);
                    bw=[bws,bws];
                elseif strcmp(bandwidth,'CCT-RBC')==1
                    output = rddcct(x,y,c);
                    bws=output.h;
                    bw=[bws,bws];
                end
                
                if length(bw)==2
                    B(:,nb)=bw';
                else
                    B(:,nb)=[bw;bw];
                end
                if strcmp(bandwidth,'AI-RBC')==1 ||strcmp(rich,'IK-RBC')==1 ||strcmp(rich,'CCT-RBC')==1
                    [Sn_hat,P_u]=FRDTest_step_AL_RBC(y,x,t,bw,c,nbs,vweight,xi,rich);
                else
                    [Sn_hat,P_u]=FRDTest_step_AL(y,x,t,bw,c,nbs,vweight,xi,rich);
                end
                pval(nb) = mean((P_u>=Sn_hat));
                Sn_vec(nb)=Sn_hat;
                P_u_sort=sort(P_u);
                CV01(nb)=P_u_sort(0.99*nbs);
                CV05(nb)=P_u_sort(0.95*nbs);
                CV10(nb)=P_u_sort(0.90*nbs);
            end
            
            
            disp(['cutoff=',num2str(40*cutoff),'  n=',num2str(n)])
            disp(VW{vweight+1})
            if vweight==1
                disp(xi)
            end
            disp(['bandwidth type',band])
            disp(B)
            disp(['    p-values        ',num2str(pval)])
            disp(['    Stat.         ',num2str(Sn_vec)])
            disp(['    1% CV.      ',num2str(CV01)])
            disp(['    5% CV.      ',num2str(CV05)])
            disp(['    10% CV.      ',num2str(CV10)])
        end
    end
end
toc
diary off
beep;
