function h_opt=rd_optbandwidth_1(y,x,threshold)
%--------------------------------------------------------------------
% PURPOSE: calculates optimal IK bandwidth for regression discontinuity
%          settings (revised from original to only output bandwidth)
%--------------------------------------------------------------------
% INPUTS
%           y is the outcome
%           x is the scalar forcing variable
%           w is the binary treatment indicator (equal to an indicator
%             that x exceeds threshold in a sharp rd design.
%           z is a vector of additional covariates (always need as argument, even if not used)
%   threshold is the value of the threshold for the forcing variable
% if output=1, print all intermediate steps, if output=0 do not
% if addcov=1, the variables in z will be used as additional covariates
% if addcov=0 they will be ignored.
%--------------------------------------------------------------------
% OUTPUT
%      h_opt is bandwidth
%--------------------------------------------------------------------

c=threshold;
n=length(y);    % number of observations
x=x-c;          % forcing variable relative to threshold
mx=mean(x);
sx=sqrt((x-mx)'*(x-mx)/(n-1));  % sample standard deviation of forcing variable

%pause

h_silverman=1.84*sx*(n^(-1/5));     % initial bandwidth
% based on silverman's rule


n_min=sum(x<0);           % number of observations to left of threshold
n_plus=sum(x>=0);         % number of observations to the right of threshold

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% optimal bandwidth                                                                %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



% ===============================================================================
% step one

i_plus_1=(x>=0)&(x<=h_silverman);   % positive observatins within bandwidth
i_min_1=(x<0)&(x>=-h_silverman);    % negative observations within bandwidth

n_plus_1=sum(i_plus_1);    % number of positive observations within bandwidth
n_min_1=sum(i_min_1);      % number of negative observations within bandwidth

y_ave_plus_1=y'*i_plus_1/n_plus_1;   % average outcome on positive side within bandwidth
y_ave_min_1=y'*i_min_1/n_min_1;      % average outcome on negative side within bandwidth

dy_plus_1=y(i_plus_1,1)-y_ave_plus_1; % residuals on positive side within bandwidth
dy_min_1=y(i_min_1,1)-y_ave_min_1;    % residuals on negative side within bandwidth

s_min_1=sqrt(dy_min_1'*dy_min_1/(n_min_1-1));     % standard dev for obs to left of threshold within bandwidth
s_plus_1=sqrt(dy_plus_1'*dy_plus_1/(n_plus_1-1)); % standard dev for obs to right of threshold within bandwidth

% estimated variance at threshold
sigmas=(dy_plus_1'*dy_plus_1+dy_min_1'*dy_min_1)/(n_plus_1+n_min_1);
% estimated standard deviation at threshold
sigma=sqrt(sigmas);

% estimated density at threshold
fc=(n_plus_1+n_min_1)/(2*n*h_silverman);  % estimate of marginal density at threshold





% ===============================================================================
% step two

median_plus=median(x(x>=0,1));
median_min=median(x(x<0,1));
middle=(x>=median_min)&(x<=median_plus);
x_middle=x(middle,1);
y_middle=y(middle,1);
tt=[ones(sum(middle),1),x_middle>=0,x_middle,x_middle.*x_middle,x_middle.*x_middle.*x_middle];
%tt=[[t_min,zeros(n_min_2,1)];[t_plus,ones(n_plus_2,1)]];
gamma=inv(tt'*tt)*(tt'*y_middle);
yhat=tt*gamma;
third_der=6*gamma(5,1);           % estimate of third derivative

% optimal bandwidth for estimating second derivatives
h_plus_2=3.56*(sigmas/(fc*max(third_der*third_der,0.01)))^(1/7)*n_plus^(-1/7);
h_min_2=3.56*(sigmas/(fc*max(third_der*third_der,0.01)))^(1/7)*n_min^(-1/7);

n_plus_2=sum(middle&(x>=0));
n_min_2=sum(middle&(x<0));

i_min_3=(x<0)&(x>=-h_min_2);     % negative values within bandwidth
if sum(i_min_3)<=10
    i_min_3=(x<0);
end
n_min_3=sum(i_min_3);
y_min_3=y(i_min_3,1);            % y-values for negative x within bandwidth
x_min_3=x(i_min_3,1);            % x-values for negative x within bandwidth
t_min=[ones(sum(i_min_3),1),x_min_3,x_min_3.*x_min_3];
% regression from the left
lambda_min=inv(t_min'*t_min)*(t_min'*y_min_3);
%' second derivative from left'
second_der_min=2*lambda_min(3,1);        % estimate of second derivative from left

i_plus_3=(x>=0)&(x<=h_plus_2);
n_plus_3=sum(i_plus_3);
if sum(i_plus_3)<=10
    i_plus_3=(x>=0);
end
y_plus_3=y(i_plus_3,1);
x_plus_3=x(i_plus_3,1);
t_plus=[ones(sum(i_plus_3),1),x_plus_3,x_plus_3.*x_plus_3];
%'lambda estimate from right'
lambda_plus=inv(t_plus'*t_plus)*(t_plus'*y_plus_3);
%' second derivative from right'
second_der_plus=2*lambda_plus(3,1);        % estimate of second derivative from right



% ===============================================================================
%'begin step 3'

%'r_plus regularization term from right'
r_plus=720*sigmas/(n_plus_3*(h_plus_2^4));
%'r_min regularization term from left'
r_min=720*sigmas/(n_min_3*(h_min_2^4));

CK=3.4375;
%'hat h opt'
h_opt=CK*((2*sigmas/(fc*((second_der_plus-second_der_min)^2+r_plus+r_min)))^(1/5))*(n^(-1/5));
% disp(CK*((2*sigmas/(fc*((second_der_plus-second_der_min)^2+r_plus+r_min)))^(1/5)))
return